###########################################################################
#   R functions to estimate the covariate coefficients (betas)            #
#   and mean regression function (eta) via Monte Carlo Simulation,        #
#   corresponding to the manuscript titled,                               #
#   "Single-index regression for pooled biomarker data"		                #
#                                                                         #
#   The estimation procedure does not require any prior knowledge         #
#   on the biomarker distribution                                         #
#                                                                         #
#   An illustrate example was given under normal biomarker distribution,  #
#   however, out estimates still performed very well under right-skew     #
#   biomarker distributions, such as Gamma or LogNormal.                  #
#                                                                         #
#                                                 Date: 09/02/2017.	      #
###########################################################################
rm(list=ls(all=TRUE))

###########################################################################
# required packages
###########################################################################
library(Rcpp)
library(RcppArmadillo)
library(MASS)
library(alabama)
###########################################################################
# Rcpp was used to compute the local linear estimator eta for a given beta
sourceCpp("SiMPool.cpp")
###########################################################################

###########################################################################
# Part I: Find the bandwidth and the initial beta via minimizing the leave-one-out 
# cross validation objective function with respect to h and beta simultaneously.
# This bandwidth selection method was originally proposed in Härdle, Hall, and Ichimura (1993).
###########################################################################
# Arguments:
# tem.X = covariate matrix
# tem.n = group size
# tem.gY = pooled responses
# Outputs:
# res.h: vector of paramater estimates including beta and h
INI=function(tem.X,tem.n,tem.gY)
{
  tem.p=ncol(tem.X)
  tem.N=nrow(tem.X)
  # tem.ind1 gives the starting index in the pool for each individual
  # tem.ind2 gives the ending index in the pool for each individual
  tem.ind1=seq(0,tem.N-tem.n,by=tem.n)+1
  tem.ind1=rep(tem.ind1,each=tem.n)
  tem.ind2=tem.ind1+tem.n-1
  tem.ind1=c(tem.ind1,rep(tem.N-tem.N%%tem.n+1,tem.N%%tem.n))
  tem.ind2=c(tem.ind2,rep(tem.N,tem.N%%tem.n))
  # construct the pooled response vector with length N
  Fake.Y=c(rep(tem.gY[1:(length(tem.gY)-ifelse(tem.N%%tem.n,1,0))],each=tem.n), 
           rep(tem.gY[length(tem.gY)],tem.N%%tem.n))
  Mu=mean(Fake.Y)
  # n.vec is the vector of group sizes
  tem.gn=matrix(c(rep(tem.n,ceiling(tem.N/tem.n)-1),ifelse(tem.N%%tem.n, tem.N%%tem.n, tem.n)),tem.N,1)
  run.Y=tem.gn*Fake.Y-(tem.gn-1)*Mu
  # S.object is the weighted least squares objective function defined in manuscript equation (2) 
  S.object=function(tem.btah)
  {
    tem.h=tem.btah[tem.p]
    tem.bta=tem.btah[-tem.p]
    tem.bta=c(sqrt(1-sum((tem.bta)^2)),tem.bta)
    tem.fit=SiMLLCV(tem.h,tem.X,matrix(run.Y),matrix(tem.bta),tem.ind1,tem.ind2)[1,]
    tem.gn=c(rep(tem.n,ceiling(tem.N/tem.n)-1),ifelse(tem.N%%tem.n, tem.N%%tem.n, tem.n))
    tem.resi=tem.gn*tem.gY-rowSums(matrix(c(tem.fit,rep(0,(tem.N%%tem.n!=0)*(tem.n-tem.N%%tem.n))),,tem.n,byrow=TRUE))
    return(sum(tem.resi^2/tem.gn))
  }
  # hin is a vector function specifying the norm=1 constraint, used in minimization of S.object. 
  hin<-function(x)
  {
    return(c(1-sum(x[-tem.p]^2),x[tem.p]))
  }
  # package alabama was adopt for constraint optimization
  res.h=auglag(c(rep(1,tem.p-1)/sqrt(tem.p),sd(tem.X)*(ceiling(tem.N/tem.n))^(-.2)),S.object,hin=hin)
  return(res.h)
}

###########################################################################
# Part II: Find the final estimate of beta by using the initial beta 
# and bandwidth obtained from Part I.
###########################################################################
# Arguments:
# tem.X = covariate matrix
# tem.n = group size
# tem.gY = pooled responses
# tem.inih = vector of initial beta and h
# Outputs:
# res: estimates of beta
FIT=function(tem.X,tem.n,tem.gY,tem.inih)
{
  tem.p=ncol(tem.X)
  tem.N=nrow(tem.X)
  Fake.Y=c(rep(tem.gY[1:(length(tem.gY)-ifelse(tem.N%%tem.n,1,0))],each=tem.n), 
           rep(tem.gY[length(tem.gY)],tem.N%%tem.n))
  Mu=mean(Fake.Y)
  tem.gn=matrix(c(rep(tem.n,ceiling(tem.N/tem.n)-1),ifelse(tem.N%%tem.n, tem.N%%tem.n, tem.n)),tem.N,1)
  run.Y=tem.gn*Fake.Y-(tem.gn-1)*Mu
  tem.h=tem.inih[tem.p]
  ini.bta=tem.inih[-tem.p]
  
  Fit.object=function(tem.bta)
  {
    tem.bta=c(sqrt(1-sum((tem.bta)^2)),tem.bta)
    tem.fit=SiMLL(tem.h,tem.X,matrix(run.Y),matrix(tem.bta),matrix(tem.X%*%tem.bta))[1,]
    tem.gn=c(rep(tem.n,ceiling(tem.N/tem.n)-1),ifelse(tem.N%%tem.n, tem.N%%tem.n, tem.n))
    tem.resi=tem.gn*tem.gY-rowSums(matrix(c(tem.fit,rep(0,(tem.N%%tem.n!=0)*(tem.n-tem.N%%tem.n))),,tem.n,byrow=TRUE))
    return(sum(tem.resi^2/tem.gn))
  }
  
  hin<-function(x)
  {
    return(1-sum(x^2))
  }
  
  res=auglag(ini.bta,Fit.object,hin=hin)
  return(res)
}

###########################################################################
# Part III: Main function
###########################################################################
# In this function, we estimate the covariate coefficients (betas) and mean regression function (eta)     
# based on the pooled responses artificially constructed using individual level data for a given group size.

# Arguments:
# n = group size
# N = number of individuals 
# X = covariate matrix
# Y = individual level responses 
main<- function(n,N,X,Y){
  J=N/n
  # calculate pooled response
  group.Y=rowMeans(matrix(c(Y,rep(sum(Y[(N-N%%n+1):N])/(N%%n),(N%%n!=0)*(n-N%%n))),,n,byrow=TRUE))
  # obtain the initial beta and bandwidth h 
  system.time({ini.btah=INI(X,n,group.Y)})
  h=(ini.btah$par)[p]
  # obtain the final estimate of beta
  system.time({Fit.bta=FIT(X,n,group.Y, ini.btah$par)})
  bta.est=c(sqrt(1-sum((Fit.bta$par)^2)),Fit.bta$par)
  # find the estimate of eta using the estimate of beta and calculate the MSE
  t=seq(-1,1,length.out = N)
  fit=SiMLL(h,X,matrix(Y),matrix(bta.est),matrix(t))[1,]
  mse= mean((eta(t)-fit)^2)
  # combine the results of beta estimate and mse 
  res=c(bta.est,mse)  
  # output the combined result and estimate of eta in a list
  return(list(res=res,resc=fit))
}

########################################################################
# An illustrative Example: 
# This is an example of the secenario where the number of individual N is fixed.
# We consider 5000 individuals with 3 covariates. 
# Two continuous covariates follow Uniform(-1,1) and Normal(0,0.09), respectively.
# One discrete covariate with half probablity to be 0.5 and the other half to be -0.5. 
# The true beta is (0.5,0.5,0.707).
# The true sigma is 0.5.
# The mean regression function eta we considered is x^2.
# The conditional distributions of the individual biomarker level Y given the covariate X is normal. 
########################################################################
# Important note
# Once users defined beta, sig, N, X, eta function, biomarker dist,
# the following codes do not need to be changed except "set.seed"
########################################################################
set.seed(10)
N=5000
p=3
# covariate information matrix 
X1<- runif(N,-1,1)
X2<- rnorm(N,0,0.3)
X3<- rbinom(N,1,0.5)-0.5
X<-matrix(c(X1,X2,X3),N,p)
# true parameters
bta<-c(0.5,0.5,1/sqrt(2))
sig<-0.5
# eta function
eta<-function(x){return(x^2)}
# biomarker distribution
Y<- eta(X%*%bta)+rnorm(N,0,sig)

#*******************************************************************#
# group size = 1
n1<-main(1,N,X,Y)
bta.hat<-n1$res[1:p];mse<-n1$res[p+1]
rbind(bta,bta.hat)
mse
# > rbind(bta,n1$res[1:p])
#              [,1]     [,2]      [,3]
# bta     0.5000000 0.5000000 0.7071068
# bta.hat 0.5869782 0.5021946 0.6350253
# > mse
# [1] 0.005482716
#*******************************************************************#
# group size = 2
n2<-main(2,N,X,Y)
bta.hat<-n2$res[1:p];mse<-n2$res[p+1]
rbind(bta,bta.hat)
mse
# > rbind(bta,bta.hat)
#              [,1]      [,2]      [,3]
# bta     0.5000000 0.5000000 0.7071068
# bta.hat 0.5478694 0.4682651 0.6932293
# > mse
# [1] 0.0009451894
#*******************************************************************#
# group size = 5
n5<-main(5,N,X,Y)
bta.hat<-n5$res[1:p];mse<-n5$res[p+1]
rbind(bta,bta.hat)
mse
# > rbind(bta,bta.hat)
#              [,1]      [,2]      [,3]
# bta     0.5000000 0.5000000 0.7071068
# bta.hat 0.5379681 0.4584699 0.7073865
# > mse
# [1] 0.0009081685
#*******************************************************************#
# group size = 10
n10<-main(10,N,X,Y)
bta.hat<-n10$res[1:p];mse<-n10$res[p+1]
rbind(bta,bta.hat)
mse
# > rbind(bta,bta.hat)
#              [,1]      [,2]      [,3]
# bta     0.5000000 0.5000000 0.7071068
# bta.hat 0.5473279 0.4231332 0.7220737
# > mse
# [1] 0.001260436

